
#include <SM.h>//state machine library
#include <SPI.h>
#include <Mcp23s17.h>//library to use MCP23S17 with 2 different CS pin
#include <Ethernet.h>
#include "Mudbus.h"

#define circuit1_SLAVE_SELECT_PIN  2 //arduino   <->   SPI Slave Select   -> CS  (Pin 11 on MCP23S17 DIP)
#define circuit2_SLAVE_SELECT_PIN  3 //arduino   <->   SPI Slave Select    -> CS  (Pin 11 on MCP23S17 DIP)

MCP23S17 circuit1 = MCP23S17( circuit1_SLAVE_SELECT_PIN, 0x01 );//addresse 1
MCP23S17 circuit2 = MCP23S17( circuit2_SLAVE_SELECT_PIN, 0x02 );//addresse 2

int etat = 0; // used for RE (RISING EDGE) and FE (FALLING EDGE)
int memo;
int M = 0;
//int AUTO;
float VALdac1 = 4095;
float VALdac2 = 1300;
word outputValue1 = 0; // a word is a 16-bit number
byte data1 = 0; // and a byte is an 8-bit number

#define BPATU 8 //emergency stop
#define DCY 9  //cycle run
#define FCY A2  //end the cycle
#define AUTO A3  //AUTO=0 AUTOMATIC mode, AUTO=1 MANUAL mode
uint16_t circuit1read;//read the I2C INPUTS circuit1
uint16_t circuit2read;//read the I2C INPUTS circuit2

/*-----( Declare objects )-----*/
SM Secu(STATE100);             //the master machine
SM RunStop(STATE20);            //the run stop machine
SM AutoManu(STATE0);            //the automatic/manual mode machine

Mudbus Mb;//modebus registers declaration

void setup()
{

  uint8_t mac[]     = { 0x90, 0xA2, 0xDA, 0x00, 0x51, 0x06 };
  uint8_t ip[]      = { 192, 168, 1, 44};
  uint8_t gateway[] = { 192, 168, 1, 1 };
  uint8_t subnet[]  = { 255, 255, 255, 0 };
  Ethernet.begin(mac, ip, gateway, subnet);
  //Avoid pins 4,10,11,12,13 when using ethernet shield
  delay(5000);
  //push buttons

  pinMode(DCY, INPUT);
  pinMode(FCY, INPUT);
  pinMode(AUTO, INPUT);
  pinMode(BPATU, INPUT);
  pinMode(5, OUTPUT); //analog SPI expander pin CS
  pinMode(6, OUTPUT); //analog SPI expander pin CS

  pinMode(circuit1_SLAVE_SELECT_PIN, OUTPUT);//digital expander pin CS
  pinMode(circuit2_SLAVE_SELECT_PIN, OUTPUT);


  //circuit1.pinMode(0,INPUT);//INTERNAL PULLUP NOT necessary here
  //  unless both input would be activated,WHEN INPUT AND OUTPUT ON THE SAME CIRCUIT PORT
  circuit1.pinMode(1, OUTPUT); //@1A1
  circuit1.pinMode(9, OUTPUT); //@1B1
  circuit1.digitalWrite(1, LOW);
  circuit1.digitalWrite(9, LOW);
  circuit1.pinMode(6, INPUT); //BP @1A6
  circuit1.pinMode(14, INPUT); //BP @1B6

  circuit2.pinMode(1, OUTPUT); //@2A1
  circuit2.pinMode(9, OUTPUT); //@2B1
  circuit2.digitalWrite(1, LOW);
  circuit2.digitalWrite(9, LOW);
  circuit2.pinMode(6, INPUT);//BP @2A6
  circuit2.pinMode(14, INPUT);//BP @2B6

  //circuit2.port(0x0000);//toutes les sorties à 0
  
  SPI.begin(); // wake up the SPI bus.
  SPI.setBitOrder(MSBFIRST);//needed to use the SPI DAC


  //spi analog to 0V, using 2x SPI DAC without any library
  outputValue1 = 0;//the same word to control both SPI DAC
  digitalWrite(5, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(5, HIGH);
  delay(50);

  outputValue1 = 0;
  digitalWrite(6, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(6, HIGH);
  delay(50);

}


void loop()
{
  circuit1read = circuit1.port(); //read the port circuit1
  circuit2read = circuit2.port(); //read the port circuit2


  Mb.Run();
  //Mb.R[60] = 12345;//address 40009 in advancedHMI
  //memo = Mb.R[8];
  //Mb.R[66] = Mb.R[8];

  EXEC(Secu);//launch Master state machine

  //emergency stop/run/stop machine
  if (digitalRead(BPATU) == 0) { //@1A1 Stops the Master machine, (emergency stop), fourth button on schematic
    RunStop.Finish();//stop the slaves
    AutoManu.Finish();
  }

  if ((digitalRead(BPATU) == 1) && RunStop.Finished && AutoManu.Finished)
  {
    EXEC(RunStop);            //Launch the slaves
    RunStop.Set(STATE20);

    EXEC(AutoManu);            //Launch the slaves
    AutoManu.Set(STATE0);
  }
  //LIGHTING the light AUTO MANUAL on the HMI via modbus registers
  if ((digitalRead(AUTO) == 1) || (Mb.R[19] == 1))  {
    Mb.R[51] = 1;//HMI LIGHT AUTO
    Mb.R[52] = 0;
  }
  else {
    Mb.R[51] = 0;//auto hmi light
    Mb.R[52] = 1;
  }


}//--(end main loop )---

State STATE100()//on emergency
{
  circuit1.digitalWrite(1, LOW);
  Mb.R[139] = 0;
  circuit1.digitalWrite(9, LOW);
  Mb.R[140] = 0;
  circuit2.digitalWrite(1, LOW);
  Mb.R[141] = 0;
  circuit2.digitalWrite(9, LOW);
  Mb.R[142] = 0;
  M = 0;
  Mb.R[49] = 1;
  Mb.R[50] = 0;
  //spi analog to 0V
  outputValue1 = 0;
  Mb.R[169] = 0;
  Mb.R[179] = 0;
  
  digitalWrite(5, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(5, HIGH);
  delay(50);

  outputValue1 = 0;
  digitalWrite(6, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(6, HIGH);
  delay(50);

  if (digitalRead(BPATU) == 1) Secu.Set(STATE101);

}

State STATE101() //off emergency, launch the slave machines
{
  Mb.R[49] = 0;
  EXEC(RunStop);
  EXEC(AutoManu);
  if (digitalRead(BPATU) == 0) Secu.Set(STATE100);

}
//---( STATE MACHINE: AutoManu modes )-----------------------------------------------------------------------------------
State STATE0()
{
  circuit1.digitalWrite(1, LOW);
  Mb.R[139] = 0;
  circuit1.digitalWrite(9, LOW);
  Mb.R[140] = 0;
  circuit2.digitalWrite(1, LOW);
  Mb.R[141] = 0;
  circuit2.digitalWrite(9, LOW);
  Mb.R[142] = 0;

  if ((Mb.R[51] == 1) && (M == 1)) AutoManu.Set(STATE1) ;

  if (Mb.R[51] == 0) AutoManu.Set(STATE5);
}

State STATE1()
{
  circuit2.digitalWrite(9, LOW);
  Mb.R[142] = 0;
  circuit1.digitalWrite(1, HIGH);
  Mb.R[139] = 1;
  if (AutoManu.Timeout(500)) AutoManu.Set(STATE2) ;
}

State STATE2()
{
  circuit1.digitalWrite(1, LOW);
  Mb.R[139] = 0;
  circuit1.digitalWrite(9, HIGH);
  Mb.R[140] = 1;
  if (AutoManu.Timeout(500)) AutoManu.Set(STATE3) ;
}

State STATE3()
{
  circuit1.digitalWrite(9, LOW);
  Mb.R[140] = 0;
  circuit2.digitalWrite(1, HIGH);
  Mb.R[141] = 1;
  if (AutoManu.Timeout(300)) AutoManu.Set(STATE4) ;
}

State STATE4()
{
  circuit2.digitalWrite(1, LOW);
  Mb.R[141] = 0;
  circuit2.digitalWrite(9, HIGH);
  Mb.R[142] = 1;
  if ((AutoManu.Timeout(500)) && M == 1) AutoManu.Set(STATE1);
  if ((AutoManu.Timeout(500)) && M == 0) AutoManu.Set(STATE0);
}

State STATE5()
{
  //bit read fuction works better
  //circuit1.digitalWrite(1,bitRead(circuit1read,6));
  //circuit1.digitalWrite(9,bitRead(circuit1read,14));
  //circuit2.digitalWrite(1,bitRead(circuit2read,6));
  //circuit2.digitalWrite(9,bitRead(circuit2read,14));
  if ((bitRead(circuit1read, 6) == 1) || (Mb.R[39] == 1))  {
    circuit1.digitalWrite(1, HIGH);
    Mb.R[139] = 1;
  }
  else {
    circuit1.digitalWrite(1, LOW);
    Mb.R[139] = 0;
  }

  if ((bitRead(circuit1read, 14) == 1) || (Mb.R[40] == 1)) {
    circuit1.digitalWrite(9, HIGH);
    Mb.R[140] = 1;
  }
  else {
    circuit1.digitalWrite(9, LOW);
    Mb.R[140] = 0;
  }

  if ((bitRead(circuit2read, 6) == 1) || (Mb.R[41] == 1)) {
    circuit2.digitalWrite(1, HIGH);
    Mb.R[141] = 1;
  }
  else {
    circuit2.digitalWrite(1, LOW);
    Mb.R[141] = 0;
  }

  if ((bitRead(circuit2read, 14) == 1) || (Mb.R[42] == 1)) {
    circuit2.digitalWrite(9, HIGH);
    Mb.R[142] = 1;
  }
  else {
    circuit2.digitalWrite(9, LOW);
    Mb.R[142] = 0;
  }


  if (Mb.R[51] == 1) AutoManu.Set(STATE0);
}

//---( STATE MACHINE: RunStop )-------------------------------------------------------------------------------
State STATE20()
{
  M = 0;
  Mb.R[50] = 0;
  outputValue1 = 0;
  Mb.R[169] = 0;
  Mb.R[179] = 0;
  if ((digitalRead(DCY) == 1) || (Mb.R[29] == 1)) RunStop.Set(STATE21) ;
}

State STATE21()
{
  M = 1;
  Mb.R[50] = 1;

  outputValue1 = Mb.R[69] * 40.9;
  Mb.R[169] = outputValue1;
  digitalWrite(5, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(5, HIGH);


  outputValue1 = Mb.R[79] * 40.9;
  Mb.R[179] = outputValue1;
  Mb.R[180] = Mb.R[79];
  digitalWrite(6, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1 = 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(6, HIGH);

  if ((digitalRead(FCY) == 1) || (Mb.R[30] == 1)) RunStop.Set(STATE20) ;
}


































































